/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * file: ui_vga.c *  purpose: handle vga graphics 
 */

#include "sins.h"
#include "ui_vga2.h"
#include <vga.h>

#define BORDERSIZE 1

/* public functions */
int ui_init ();
int init_graphics (ARENA *);
void finish_graphics (void);
void draw_arena (ARENA *);
void draw_message (char *);
void updatescreen (void);
char *prompt (char *);
int getkey (void);

int CMD_loadpictset (char *line);

/* private functions */
static void draw_borders(ARENA *);
static void draw_box (int sx, int sy, int color);
static void draw_pict (int sx, int sy, int width, int height, int *pict);
static int *load_pictset (char *filename, int width, int height);

/* private data */
static ARENA oldarena;
static int spriteheight;
static int spritewidth;
static FILE *outputfile;
int *pictset;

/* public data */
int screen_initialized = 0;

/* Initialize UI module */
int
ui_init ()
{
  /* open message tty */
  outputfile = fopen("/tmp/out", "w");
  if ( !outputfile ) return 0;
  
  if (!register_command ("loadpictset", &CMD_loadpictset,
			 "loadpictset [<width> <height> <fname>]")) return 0;

  pictset = defpictset;

  return 1;
}

/* initialize the graphics */
int
init_graphics (ARENA *arena)
{

  int xdim, ydim;
  int vmode;

  /* what video mode should we use ? */
  vmode = vga_getdefaultmode();
  if ( !vga_hasmode(vmode) )
  {
    /* find out higher supported vmode */
    for (vmode=vga_lastmodenumber(); vmode; vmode--)
    {
      if (vga_hasmode(vmode)) break;
    }
  }

  /* no mode found (weird) */
  if (!vmode)
  {
    message("No videomode supported (?)");
    finish();
  }

  /*vmode = 1;*/
  message("VGA videomode set to %i", vmode);

  /* initialize vga screen */
  vga_init();
  vga_setmode(vmode);

  xdim = vga_getxdim();
  ydim = vga_getydim();

  /* Set arena dimensions */
  if ( arena->lines || arena->cols ) message("ignoring -l and -c switches");
  if ( !spritewidth ) spritewidth=8;
  if ( !spriteheight ) spriteheight=8;
  arena->lines = ydim/spriteheight - 2*BORDERSIZE;
  arena->cols = xdim/spritewidth - 2*BORDERSIZE;

  /* initialize the oldarena (used for optimization purposes) */
  oldarena.lines = arena->lines;
  oldarena.cols = arena->cols;
  oldarena.init = 1; /* useless ? */
  oldarena.board = (sprite **)safe_malloc(
    oldarena.lines*oldarena.cols*sizeof(sprite),
    "ui_curses: draw_arena: oldarena.board");
  memset(oldarena.board, 0, oldarena.lines*oldarena.cols);

  draw_borders(arena);

  /* Set screen initialized flag */
  screen_initialized = 1;

  return 1;

}

/* vga version of message drawing function */
void
draw_message (char *str)
{
  fprintf(outputfile, "%s\n", str);
  fflush(outputfile);
  return;
}

/* arena drawing */
/**** DEFAULT SVGALIB PALETTE 
0      - black.
1      - blue.
2      - green.
3      - cyan.
4      - red.
5      - magenta.
6      - brown.
7      - grey.
8      - dark grey (light black).
9      - light blue.
10     - light green.
11     - light cyan.
12     - light red.
13     - light magenta.
14     - yellow (light brown).
15     - white (light grey).
***************************************************/
void
draw_arena (ARENA *arena)
{
  int y, x;
  sprite *sp;
  int look;

  for (y = 0; y < arena->lines; y++)
  {
    for (x = 0; x < arena->cols; x++)
    {

      sp = GETSPRITE(arena,y,x);

      /* no changes since last drawing */
      if ( (sp==GETSPRITE(&oldarena,y,x)) ) continue;

      /* update oldarena struct */
      oldarena.board[y*oldarena.cols+x] = sp;


      if (sp) switch (sp->type)
      {
        case SNAKE:
          look = PICT_SNAKE0+((snakebody *)(sp))->self->playernum;
          break;  
        case EXSNAKE:
          look = PICT_EXSNAKE;
          break; 
        case FRUIT:
          look = PICT_FRUIT0+(((fruit *)(sp))->value)-48;
          break;
        case WALL:
          look = PICT_WALL;
          break;
        default:
          look = PICT_UNKNOWN;
          break;
      }
      else look = PICT_EMPTY;

      draw_pict(
        (x+BORDERSIZE)*spritewidth,
        (y+BORDERSIZE)*spriteheight,
        spritewidth, spriteheight, pictset+look*spritewidth*spriteheight);
    
      /*draw_box(x+BORDERSIZE, y+BORDERSIZE, bgcolor);*/
    }
  }
}



/*
 * update the screen 
 */
void
updatescreen ()
{
/*
  static int motionidx=0;
  vga_drawpixel(0, motionidx++);
*/
}

void
finish_graphics ()
{
  fclose(outputfile);
  vga_setmode(TEXT);
}


/*
 * Get next key in input buffer
 */
int
getkey ()
{
  int ret;
  
  ret=vga_getkey();
  if (!ret) return -1;
  return ret;
}

/*
 * prompt the user for a string
 * the line returnered is allocated with malloc(3)
 * so the caller must free it when finished with it. 
 */
char *
prompt (char *prompt_string)
{
  return NULL;
}

int
CMD_loadpictset (char *line)
{
  char fname[256];
  int ret = 0;
  int height, width;
  int *newset;

  if ( line ) ret = sscanf (line, "%i %i %s", &width, &height, fname);
  if ( ret != 3 )
  {
    /* set default pictset */
    pictset = defpictset;
    return 1;
  }

  spritewidth = width;
  spriteheight = height;

  newset = load_pictset(fname, spritewidth, spriteheight);
  if ( !newset )
  {
    message("can't load pictset from %s", fname);
    return 0;
  }

  if ( pictset != defpictset ) free(pictset);
  pictset = newset;

  message("pictset %s loaded", fname);
  return 1;
}

/************ PRIVATE FUNCTIONS **********************/

static void
draw_borders (ARENA *arena)
{
  int x,y,i;
  int bordercolor = 7;


  for (x=0; x<arena->cols+2*BORDERSIZE; x++)
  {
      for (i=0; i<BORDERSIZE; i++)
      {
        draw_box(x,i,bordercolor);
        draw_box(x,arena->lines+2*BORDERSIZE-i-1,bordercolor);
      }
  }

  for (y=0; y<arena->lines+2*BORDERSIZE; y++)
  {
      for (i=0; i<BORDERSIZE; i++)
      {
        draw_box(i,y,bordercolor);
        draw_box(arena->cols+2*BORDERSIZE-i-1,y,bordercolor);
      }
  }
}

static void
draw_box (int sx, int sy, int color)
{
  int x,y,i,j;
  int white = vga_white();

  vga_setcolor(color);
  for (x=sx*spritewidth, i=0; i<spritewidth; i++, x++)
  {
    for (y=sy*spriteheight, j=0; j<spriteheight; y++, j++)
    {
      vga_drawpixel(x,y);
    }
  }
  vga_setcolor(white);
}

static void
draw_pict (int sx, int sy, int width, int height, int *pict)
{
  int x,y,i,j;
  int white = vga_white();

  for (x=sx, i=0; i<width; i++, x++)
  {
    for (y=sy, j=0; j<height; y++, j++)
    {
      vga_setcolor(*(pict+(j*width)+i));
      vga_drawpixel(x,y);
    }
  }
  vga_setcolor(white);
}

static int *
load_pictset (char *filename, int width, int height)
{
  FILE *stream;
  int idx = 0;
  int quoted = 0;
  int ch;
  int *set;

  stream = fopen(filename, "r");
  if ( !stream ) return NULL;

  if ( screen_initialized && (width != spritewidth || height != spriteheight) )
  {
    message("Can't change sprite size while running");
    return NULL;
  }

  spritewidth = width;
  spriteheight = height;
  set = safe_malloc(width*height*PICTSETLEN*sizeof(int), "New pictset");

  while (!feof(stream))
  {
    if ( idx >= width*height*PICTSETLEN ) break;
    ch=getc(stream);
    if (ch) switch(ch)
    {
      case '\n':
      case '\r':
        quoted=0;
        break;
      case ' ':
      case '\t':
        break;
      case '#':
        quoted=1;
        break;
      default:
        if ( !quoted )
        {
          *(set+idx) = ch-97;
          idx++;
        }
        break;
    }
  }
  fclose(stream);
  return set;
}
